@@ -1,9 +1,9 @@
 import construct
 
-from mercury_engine_data_structures.formats import BaseResource, game_model_root
+from mercury_engine_data_structures.formats import BaseResource, standard_format
 from mercury_engine_data_structures.game_check import Game
 
-BREM = game_model_root.create('CEnvironmentMusicPresets', 0x02000004)
+BREM = standard_format.game_model('CEnvironmentMusicPresets', 0x02000004)
 
 
 class Brem(BaseResource):
