@@ -1,9 +1,9 @@
 import construct
 
-from mercury_engine_data_structures.formats import BaseResource, game_model_root
+from mercury_engine_data_structures.formats import BaseResource, standard_format
 from mercury_engine_data_structures.game_check import Game
 
-BRES = game_model_root.create('CEnvironmentSoundPresets', 0x02020001)
+BRES = standard_format.game_model('CEnvironmentSoundPresets', 0x02020001)
 
 
 class Bres(BaseResource):
