@@ -10,7 +10,7 @@ from mercury_engine_data_structures.game_check import Game
 TXT = Struct(
     "magic" / Const(b'BTXT'),
     "version" / Const(b'\x01\x00\x0a\x00'),
-    "strings" / DictAdapter(GreedyRange(DictElement(CStringRobust("utf16"))))
+    "strings" / DictAdapter(GreedyRange(DictElement(CStringRobust("utf-16-le"))))
 )
 
 class Txt(BaseResource):
